@@ -2,10 +2,10 @@ package Net::DNS::Resolver::MSWin32;
 use base Net::DNS::Resolver::Base;
 
 #
-# $Id: MSWin32.pm 1021 2012-10-04 09:18:29Z willem $
+# $Id: MSWin32.pm 1045 2012-11-09 11:03:03Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1021 $)[1];
+$VERSION = (qw$LastChangedRevision: 1045 $)[1];
 
 =head1 NAME
 
@@ -42,7 +42,7 @@ sub init {
 	my @nameservers = map { $_->{IpAddress} } @{$FIXED_INFO->{DnsServersList}};
 
 	my %h;							# remove blanks and dupes
-	$defaults->{nameservers} = [grep { !$_ || $h{$_}++ } @nameservers] if scalar @nameservers;
+	$defaults->{nameservers} = [grep { $_ && !$h{$_}++ } @nameservers] if scalar @nameservers;
 
 	my $domain = $FIXED_INFO->{DomainName} || '';
 	my $searchlist = $domain;
